using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.Primitives;
using Telerik.WinControls.UI;
using Telerik.WinControls.Commands;


namespace Telerik.WinControls.UI
{
    public class RadCarouselElement: RadItem
    {
        private RadItemOwnerCollection items;
        private int selectedIndex = -1;
        bool arranged = false;
        private TextPrimitive textElement;
        private TextPrimitive textTitleElement;

        private Size textDisplaySize = new Size(200, 200);
        private Size maxItemSize = new Size(240, 180);

        private Size calculatedTextDisplaySize;
        private int caruselYRadius = 60;
        
        private RadButtonElement btnLeft;
        private RadButtonElement btnRight;
        private FillPrimitive spotLight;
        private FillPrimitive backFill;
        private RadCarouselLayout layout;

        public delegate void ExampleStartEventHandler ( object sender, string selectedExampleText );
        public event ExampleStartEventHandler ExampleStart;

        
        public RadCarouselElement()
        {
            items = new RadItemOwnerCollection();
            items.ItemTypes = new Type[] { typeof( RadCarouselItem ) };
            items.ItemsChanged += new ItemChangedDelegate(items_ItemsChanged);
        }

        void items_ItemsChanged(RadItemCollection changed, RadItem target, ItemsChangeOperation operation)
        {
            if (operation == ItemsChangeOperation.Inserted ||
                operation == ItemsChangeOperation.Set)
            {
                target.AutoSize = false;

                target.Bounds = Rectangle.Empty;
                target.Visibility = ElementVisibility.Hidden;

                if (target is RadCarouselItem)
                {
                    int targetIndex = changed.IndexOf(target);
                    RadCarouselItem button = target as RadCarouselItem;
                    //button.ForeColor = Color.White;
                    //button.TextAlignment = ContentAlignment.MiddleCenter;
                    button.ShowBorder = false;

                    FillPrimitive fill = button.ButtonFillElement as FillPrimitive;
                    if (fill != null)
                    {
                        CarouselMouseOverBehavior behavior = new CarouselMouseOverBehavior(Color.FromArgb(234, 244, 255));
                        behavior.SetupInitialItemState(fill);
                        fill.AddBehavior(behavior);
                        fill.PositionOffset = new SizeF(0f, -20f);
                    }

                    target.AddBehavior(new CarouselItemCursorBehavior(this));
                    //button.Shape = new RoundRectShape(5);
                }

                if (operation == ItemsChangeOperation.Inserted)
                {
                    if (this.SelectedIndex == -1)
                        this.SelectedIndex = 0;
                }
            }
            //reset selected index whenever items changed
            else if (operation == ItemsChangeOperation.Cleared || 
                operation == ItemsChangeOperation.Removed)
            {
                this.SelectedIndex = -1;
            }

            arranged = false;
        }
        
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(true), Category(RadDesignCategory.DataCategory)]
        public RadItemOwnerCollection Items
        {
            get
            {
                EnsureChildElements();
                return this.items;
            }
        }

        [DefaultValue(-1)]
        public int SelectedIndex
        {
            get
            {
                return selectedIndex;
            }
            set
            {
                if (this.Items.Count > 0 && value > this.Items.Count - 1)
                {
                    throw new IndexOutOfRangeException("SelectedIndex");
                }

                if (selectedIndex != value)
                {
                    selectedIndex = value;
                    
                    this.arranged = false;
                    this.Invalidate(false);
                    //this.Rearange();
                }
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TextPrimitive TextElement
        {
            get
            {
                this.EnsureChildElements();
                return this.textElement;
            }
        }


        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public TextPrimitive TextTitleElement
        {
            get
            {
                this.EnsureChildElements();
                return this.textTitleElement;
            }
        }

        [DefaultValue("200, 200")]
        public Size TextDisplaySize
        {
            get { return textDisplaySize; }
            set { textDisplaySize = value; }
        }

        [DefaultValue("240, 180")]
        public Size MaxItemSize
        {
            get { return maxItemSize; }
            set { maxItemSize = value; }
        }

        [DefaultValue(60)]
        public int CaruselYRadius
        {
            get { return caruselYRadius; }
            set { caruselYRadius = value; }
        }

        //private void ApplyThemeToButton(RadItem item, string themeName)
        //{
        //    DefaultStyleBuilder builder = ThemeResolutionService.GetStyleSheetBuilder(this.ElementTree.Control, item.GetThemeEffectiveType().FullName, string.Empty, themeName)
        //                                  as DefaultStyleBuilder;

        //    if (builder != null)
        //        item.Style = new XmlStyleSheet(builder.Style).GetStyleSheet(); //clone because control might modify it later
        //}

        //protected override void ApplyTheme()
        //{
        //    ApplyThemeToButton(this.btnLeft, "OfficeGlass");
        //    ApplyThemeToButton(this.btnRight, "OfficeGlass");

        //    base.ApplyTheme();
        //}

        private Point CalcPoint(Point center, int radiusX, int radiusY, float z, int angleGrad)
        {
            double rad = Math.PI/180*angleGrad;

            int x = (int)Math.Round(center.X + Math.Sin(rad) * radiusX);
            double fY = center.Y + Math.Cos(rad) * radiusY;

            int y = (int)Math.Round(fY / 2F  + (center.Y + radiusY) * z * 0.65);

            return new Point(x, y);
        }

        private void Rearrange()
        {
            if (this.Size == Size.Empty)
            {
                return;
            }            

            this.EnsureChildElements();

            if (this.Items.Count < 2)
            {
                this.btnLeft.Visibility = ElementVisibility.Collapsed;
                this.btnRight.Visibility = ElementVisibility.Collapsed;
            }
            else
            {
                this.btnLeft.Visibility = ElementVisibility.Visible;
                this.btnRight.Visibility = ElementVisibility.Visible;
            }

            //arrange scene            
            this.backFill.Bounds = new Rectangle(0, 0, 20, this.Size.Height);
            this.spotLight.Bounds = new Rectangle((int)((this.Size.Width - this.Size.Width * 1.3) / 2) , (int)(-this.Size.Height * 1.8 / 2), (int)(this.Size.Width * 1.4), (int)(this.Size.Height * 1.8));

            //arange visible items
            if (this.SelectedIndex < 0 || this.Items.Count == 0)
                return;            

            int visibleItemCount = this.Items.Count;

            //distance between the items (angle degrees)
            float itemDistance = 360f / visibleItemCount;

            int centerIndex = visibleItemCount / 2;

            int itemIndex = this.SelectedIndex;
            int offsetIndex = 0;

            this.calculatedTextDisplaySize = new Size(
                Math.Max(this.TextDisplaySize.Width, this.Size.Width - 20),
                this.TextDisplaySize.Height);

            Point centerPoint = new Point(this.Size.Width / 2, (int)(this.Size.Height / 2.5 - TextDisplaySize.Height / 2));

            //int reflectionHeight = maxItemSize.Height / 3;

            Size calculatedMaxItemSize = maxItemSize;//new Size(maxItemSize.Width, maxItemSize.Height + reflectionHeight);

            int radiusY = caruselYRadius; //bottomBasePoint.Y - centerPoint.Y - TextDisplaySize.Height/* / 2*/;
            int radiusX = (int)(this.Size.Width / 2.0f) - calculatedMaxItemSize.Width / 2;

            Point bottomBasePoint = new Point(centerPoint.X, centerPoint.Y + radiusY);

            float minScaleF = 0.5f;

            ArrangeNavigationButtons(radiusX, radiusY, new Point(centerPoint.X, bottomBasePoint.Y));

            //loop through items to rearange
            for (int i = 0; i < visibleItemCount; i++)
            {
                RadItem item = this.Items[itemIndex];

                if (item.Bounds == Rectangle.Empty) //set initial item position to control center
                {
                    item.Bounds = new Rectangle(
                         centerPoint.X - calculatedMaxItemSize.Width / 2,
                         centerPoint.Y - calculatedMaxItemSize.Height / 2,
                         calculatedMaxItemSize.Width / 2,
                         calculatedMaxItemSize.Height / 2);                    
                }

                item.Visibility = ElementVisibility.Visible;

                item.ZIndex = -Math.Abs(offsetIndex);
                float scaleF = 1f - (((1f - minScaleF) / (visibleItemCount / 2f)) * Math.Abs(offsetIndex));

                Point itemPoint = this.CalcPoint(centerPoint, radiusX, radiusY, scaleF, (int)(itemDistance * offsetIndex));

                int itemWidth = (int)(calculatedMaxItemSize.Width * scaleF);
                int itemHeight = (int)(calculatedMaxItemSize.Height * scaleF);

                itemPoint = Point.Add(itemPoint, new Size(-itemWidth / 2, -itemHeight / 2));

                BeginItemAnimation(i, item, itemPoint, calculatedMaxItemSize, scaleF, visibleItemCount, bottomBasePoint);

                itemIndex++;
                if (itemIndex >= items.Count)
                {
                    itemIndex = 0;
                }

                if (offsetIndex == centerIndex)
                {
                    if (this.Items.Count % 2 == 0)
                        offsetIndex = -centerIndex + 1;
                    else
                        offsetIndex = -centerIndex;
                }
                else
                    offsetIndex++;
            }

            arranged = true;
        }

        private void ArrangeNavigationButtons(int radiusX, int radiusY, Point zeroPoint)
        {
            Size btnSize = new Size(45, 45);

            this.btnLeft.ZIndex = this.Items.Count + 1;
            this.btnRight.ZIndex = this.Items.Count + 1;

            int centerOffset = Math.Max((int) (radiusX * 9f / 10f), (int)(this.Size.Width / 3.8f + 10));

            int zeroY = zeroPoint.Y + maxItemSize.Height / 3;

            this.btnLeft.Bounds = new Rectangle( new Point(
                zeroPoint.X - centerOffset - btnSize.Width,
                zeroY),
                btnSize);

            this.btnRight.Bounds = new Rectangle( new Point(
                zeroPoint.X + centerOffset,
                zeroY),
                btnSize);
        }

        private void BeginItemAnimation(int i, RadItem item, Point itemPoint, Size maxItemSize, float scaleF, int visibleItemCount, Point zeroPoint)
        {
            AnimatedPropertySetting setting = new AnimatedPropertySetting(
                RadElement.BoundsProperty,
                new Rectangle(itemPoint.X, itemPoint.Y, maxItemSize.Width, maxItemSize.Height), //end value
                30,  //number of frames
                40); //interval between frames

            AnimatedPropertySetting setting1 = new AnimatedPropertySetting(
                VisualElement.OpacityProperty,
                0.4d,
                1d,
                30,
                40);

            AnimatedPropertySetting setting2 = new AnimatedPropertySetting(
                RadElement.ScaleTransformProperty,
                new SizeF(scaleF, scaleF),
                30,
                40);

            setting.ApplyEasingType = RadEasingType.OutQuad;
            setting1.ApplyEasingType = RadEasingType.InSine;
            setting2.ApplyEasingType = RadEasingType.OutQuad;

            if (i == 0)
            {
                setting.AnimationStarted +=
                    delegate
                        {
                            AnimatedPropertySetting animatedTextAppearance = new AnimatedPropertySetting
                                (
                                VisualElement.OpacityProperty,
                                0d, //end value
                                5, //number of frames
                                40 //interval between frames
                                );

                            animatedTextAppearance.ApplyValue(this.TextElement);
                            animatedTextAppearance.RemoveAfterApply = true;
                        };
            }

            if (i == visibleItemCount - 1)
            {
                BeginTextAnimation(setting, zeroPoint);
            }

            setting.ApplyValue(item);
            setting.RemoveAfterApply = true;
            setting1.ApplyValue(item);
            setting1.RemoveAfterApply = true;
            setting2.ApplyValue(item);
            setting2.RemoveAfterApply = true;
        }

        private void BeginTextAnimation(AnimatedPropertySetting setting, Point zeroPoint)
        {
            setting.AnimationFinished += (
                delegate
                    {
                        //if (this.SelectedIndex > this.Items.Count - 1)
                        //{
                        //    return;
                        //}

                        // DK_03_09_2007 - fixes some invalid index values
                        if (this.SelectedIndex < 0 ||
                            ((this.Items.Count - 1) < this.SelectedIndex))
                        {
                            return;
                        }

                        this.TextElement.Visibility = ElementVisibility.Visible;

                        this.TextElement.Bounds = new Rectangle(
                            Math.Max(30, zeroPoint.X - calculatedTextDisplaySize.Width / 2),
                            zeroPoint.Y + calculatedTextDisplaySize.Height,
                            calculatedTextDisplaySize.Width,
                            calculatedTextDisplaySize.Height);

                        this.TextTitleElement.Bounds = new Rectangle(
                            Math.Max(30, zeroPoint.X - calculatedTextDisplaySize.Width/2),
                            this.TextElement.Location.Y - 50,
                            calculatedTextDisplaySize.Width,
                            calculatedTextDisplaySize.Height);

                            this.TextElement.Text = ((RadCarouselItem) this.Items[this.SelectedIndex]).Description;
                            this.textTitleElement.Text = ((RadCarouselItem)this.Items[this.SelectedIndex]).Title;                            


                        AnimatedPropertySetting animatedTextAppearance = new AnimatedPropertySetting
                            (
                            VisualElement.OpacityProperty,
                            1d, //end value
                            30, //number of frames
                            40  //interval between frames
                            );

                        animatedTextAppearance.ApplyValue(this.TextElement);
                        animatedTextAppearance.RemoveAfterApply = true;
                    });
        }

        protected override void CreateChildElements()
        {
            backFill = new FillPrimitive();
            backFill.Class = "BackFill";
            backFill.AutoSize = false;

            this.Children.Add(backFill);

            this.spotLight = new FillPrimitive();
            spotLight.AutoSize = false;
            spotLight.Class = "SpotLight";

            this.Children.Add(spotLight);

            layout = new RadCarouselLayout();
            
            this.textElement = new TextPrimitive();

            this.textElement.ForeColor = Color.White;
            this.textElement.Visibility = ElementVisibility.Hidden;
            this.textElement.AutoSizeMode = RadAutoSizeMode.FitToAvailableSize;
            this.textElement.TextAlignment = ContentAlignment.TopLeft;
            this.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
            this.TextElement.AutoSize = false;

            layout.Children.Add(this.textElement);


            this.textTitleElement = new TextPrimitive();

            this.textTitleElement.ForeColor = Color.White;
            this.textTitleElement.Visibility = ElementVisibility.Hidden;
            this.textTitleElement.AutoSizeMode = RadAutoSizeMode.FitToAvailableSize;
            this.textTitleElement.TextAlignment = ContentAlignment.TopLeft;
            this.textTitleElement.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;
            this.textTitleElement.AutoSize = false;

            this.textTitleElement.BindProperty(VisualElement.OpacityProperty, this.textElement,
                                               VisualElement.OpacityProperty, PropertyBindingOptions.OneWay);

            this.textTitleElement.BindProperty(RadElement.VisibilityProperty, this.textElement,
                                               RadElement.VisibilityProperty, PropertyBindingOptions.OneWay);

            layout.Children.Add(this.textTitleElement);

            btnLeft = new RadButtonElement();
            SetupNavButton(btnLeft, "Telerik.QuickStart.WinControls.Resources.Carousel.left_arrow.png");
            //btnLeft.ButtonFillElement.PositionOffset = new SizeF(-5, -5);
            layout.Children.Add(btnLeft);

            btnRight = new RadButtonElement();
            SetupNavButton(btnRight, "Telerik.QuickStart.WinControls.Resources.Carousel.right_arrow.png");
            //btnRight.ButtonFillElement.PositionOffset = new SizeF(5, -5);
            layout.Children.Add(btnRight);            

            this.Children.Add(layout);

            this.Items.Owner = layout;
        }

        private void SetupNavButton(RadButtonElement button, string imageResourceName)
        {
            button.Image = this.ImageFromResource(imageResourceName);
            button.DisplayStyle = DisplayStyle.Image;
            CarouselMouseOverBehavior behavior = new CarouselMouseOverBehavior(Color.FromArgb(235, 255, 243));
            behavior.SetupInitialItemState((FillPrimitive)button.ButtonFillElement);
            button.ButtonFillElement.AddBehavior(behavior);
            button.AddBehavior(new RadMouseOverCursorBehavior(Cursors.Default, Cursors.Hand));
            button.AutoSize = false;
            button.ShowBorder = false;
        }


        private Image ImageFromResource(string resourceName)
        {
            using (Stream stream = this.GetType().Assembly.GetManifestResourceStream(resourceName))
            {
                if (stream != null)
                {
                    return new Bitmap(Image.FromStream(stream));
                }
            }

            return null;
        }

        public override void OnBubbleEvent(RadElement sender, RoutedEventArgs args)
        {
            if (args.RoutedEvent == RadElement.MouseClickedEvent)
            {
                int newSelectedIndex = -1;

                if (sender == this.btnLeft)
                {
                    newSelectedIndex = this.SelectedIndex - 1;
                    if (newSelectedIndex < 0)
                        newSelectedIndex = this.Items.Count - 1;
                } 
                else if (sender == this.btnRight)
                {
                    newSelectedIndex = this.SelectedIndex + 1;
                    if (newSelectedIndex > this.Items.Count - 1)
                        newSelectedIndex = this.Items.Count > 0? 0 : -1;
                }
                else if (sender is RadItem)
                {
                    newSelectedIndex = this.Items.IndexOf(sender as RadItem);
                    
                }

                if (newSelectedIndex >= 0)
                {
                    if (this.SelectedIndex == newSelectedIndex)
                    {
                        this.StartTransition();
                    }
                    this.SelectedIndex = newSelectedIndex;
                }
            }
        }

        private void StartTransition()
        {
            foreach(Control childControl in this.ElementTree.Control.Controls)
            {
                childControl.Visible = false;
            }
            FillPrimitive effect = new FillPrimitive();
            effect.AutoSize = false;
            effect.StretchHorizontally = false;
            effect.StretchVertically = false;

            effect.BackColor = Color.Transparent;
            effect.BackColor2 = Color.Transparent;
            effect.BackColor3 = Color.Transparent;
            effect.BackColor4 = Color.Transparent;

            effect.NumberOfColors = 2;
            effect.GradientStyle = GradientStyles.Radial;
            effect.ZIndex = 1000;

            this.layout.Children.Add(effect);
            effect.Bounds = new Rectangle(-150, -150, this.Size.Width + 300, this.Size.Height + 300);

            AnimatedPropertySetting setting = new AnimatedPropertySetting(
                FillPrimitive.BackColorProperty,
                Color.White,
                5,
                34
                );

            AnimatedPropertySetting setting1 = new AnimatedPropertySetting(
                FillPrimitive.BackColor2Property,
                Color.White,
                3,
                34
                );

            setting.AnimationFinished +=
                delegate
                    {
                        this.layout.Children.Remove(effect);
                        this.layout.Invalidate();
                        this.RaiseSelectedItemClecked();
                        foreach (Control childControl in this.ElementTree.Control.Controls)
                        {
                            childControl.Visible = true;
                        }
                    };

            setting.ApplyEasingType = RadEasingType.InQuad;
            setting.ApplyValue(effect);
            setting1.ApplyValue(effect);
        }

        private void RaiseSelectedItemClecked()
        {
            RadCarouselItem item = this.Items[this.selectedIndex] as RadCarouselItem;

            if (item == null)
            {
                return;
            }

            this.OnExampleStart( item.TreeNodeText );
        }

        
        public void OnExampleStart(string treenodeText)
        {
            if( this.ExampleStart!=null )
            {
                this.ExampleStart( this, treenodeText );
            }
        }

        protected override void OnBoundsChanged(RadPropertyChangedEventArgs e)
        {
            this.arranged = false;

            base.OnBoundsChanged(e);
        }

        protected override void PaintOverride(Telerik.WinControls.Paint.IGraphics screenRadGraphics, Rectangle clipRectangle, float angle, SizeF scale, bool useRelativeTransformation)
        {
            base.PaintOverride(screenRadGraphics, clipRectangle, angle, scale, useRelativeTransformation);

            screenRadGraphics.DrawRectangle(new Rectangle(Point.Empty, this.Size), this.BackColor);
        }

        private delegate void CallMethodDelegate();

        protected override void PaintChildren(Telerik.WinControls.Paint.IGraphics graphics, Rectangle clipRectange, float angle, SizeF scale, bool useRelativeTransformation)
        {
            if (!arranged)
            {
                arranged = true;
                this.ElementTree.Control.BeginInvoke(new CallMethodDelegate(Rearrange));
            }            

            base.PaintChildren(graphics, clipRectange, angle, scale, useRelativeTransformation);            
        }
    }
}
